// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  crystal growth group                                                  ==
// ==                                                                        ==
// ==  Stiftung caesar                                                       ==
// ==  Ludwig-Erhard-Allee 2                                                 ==
// ==  53175 Bonn                                                            ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  http://www.caesar.de/cg/AMDiS                                         ==
// ==                                                                        ==
// ============================================================================

/** \file GNUPlotWriter.h */

#ifndef AMDIS_GNUPLOTWRITER_H
#define AMDIS_GNUPLOTWRITER_H

#include "MemoryManager.h"
#include "FileWriter.h"
#include <vector>
#include <string>

namespace AMDiS {

  template<typename T> class DOFVector;
  class FiniteElemSpace;

  /** \brief
   *
   */
  class GNUPlotWriter : public FileWriterInterface
  {
  public:
    MEMORY_MANAGED(GNUPlotWriter);

    /** \brief
     * Constructor
     */
    GNUPlotWriter(std::string filename,
		  FiniteElemSpace *feSpace,
		  std::vector<DOFVector<double>*> &dofVectors);

    /** \brief
     * Destructor
     */
    virtual ~GNUPlotWriter() {};

    /** \brief
     * Implementation of FileWriter::writeFiles()
     */
    virtual void writeFiles(AdaptInfo *adaptInfo, bool force);

  protected:
    /** \brief
     * Contains the mesh
     */
    FiniteElemSpace *feSpace_;

    /** \brief
     * vector of dof vectors to write
     */
    std::vector<DOFVector<double>*> dofVectors_;

    /** \brief
     * file name
     */
    std::string filename_;
  };

}

#endif // AMDIS_VALUEWRITER_H
