#ifndef AMDIS_GAUSSELIMINATION_H
#define AMDIS_GAUSSELIMINATION_H

#include "DirectSolverInterface.h"
#include "Global.h"
#include "MemoryManager.h"

namespace AMDiS
{
  template<typename MatrixType, typename VectorType>
  class GaussElimination : public DirectSolverInterface<MatrixType, VectorType>
  {
  public:
    MEMORY_MANAGED(GaussElimination<MatrixType COMMA VectorType>);

    GaussElimination() {};

    virtual ~GaussElimination() {};

    void solve(MatrixType &a, VectorType &b, VectorType &x, int n) 
    {
      FUNCNAME("GaussElimination::solve()");

      int i, j, k, row;
      double a_ji, max, tmp = 0.0;

      // forward elimination
      for(i = 0; i < n - 1; i++) {

	// pivoting
	max = abs(a[i][i]);
	row = i;

	for(j = i + 1; j < n; j++) {
	  if(abs(a[j][i]) > max) {
	    max = abs(a[j][i]);
	    row = j;
	  }
	}

	TEST_EXIT(max > 0.0)("pivoting failed (max entry = 0)\n");

	if(row != i) {
	  tmp = b[i];
	  b[i] = b[row];
	  b[row] = tmp;
	  for(j = 0;  j < n; j++) {
	    tmp = a[i][j];
	    a[i][j] = a[row][j];
	    a[row][j] = tmp;
	  }
	}
	  
	// make zeros
	for(j = i + 1; j < n; j++) {
	  a_ji = (a[j][i] /= a[i][i]);
	  for(k = i; k < n; k++) {
	    a[j][k] -= a_ji * a[i][k];
	  }
	  b[j] -= a_ji * b[i];
	}
      } 

      // backward elimination
      for(i = n - 1; i >= 0; i--) {
	for(j = i + 1; j < n; j++) {
	  b[i] -= a[i][j] * x[j];
	}
	x[i] = b[i] / a[i][i];
      }
    };
  };
}

#endif
